<?php
/*
 * copyright (c) 2009 MDBitz - Matthew John Denton - mdbitz.com
 *
 * This file is part of HarvestAPI.
 *
 * HarvestAPI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HarvestAPI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HarvestAPI. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * HarvestAPI
 *
 * This file contains the class HarvestAPI
 *
 * @author Matthew John Denton <matt@mdbitz.com>
 * @version 0.1
 * @package com.mdbitz.harvest
 */

/**
 * HarvestAPI defines the methods available to the API, as well as 
 * handlers for parsing the returned data.
 *
 * @package com.mdbitz.harvest
 */
final class HarvestAPI{
	
	/**
	 * Harvest User
	 * @access private
	 * @var string
	 */
    private $_user;

	/**
	 * Harvest Password
	 * @access private
	 * @var string
	 */
	private $_password;
	
	/**
	 * Harvest Account
	 * @access private
	 * @var string
	 */
	private $_account;
	
	/**
     * Path to Harvest root
     *
     * @var string $path            harvest root directory
     */
    private static $_path;
	
	/**
	 * set Harvest User
	 * @param string $user
	 * @return void
	 */
	public function setUser( $user ) {
		$this->_user = $user;
	}
	
	/**
	 * set Harvest Password
	 * @param string $password
	 * @return void
	 */
	public function setPassword( $password ) {
		$this->_password = $password;
	}
	
	/**
	 * set Harvest Account
	 * @param string $account
	 * @return void
	 */
	public function setAccount( $account ) {
		$this->_account = $account;
	}
	
	/*--------------------------------------------------------------*/
	/*--------------------- Time Tracking API ----------------------*/
	/*--------------------------------------------------------------*/

	/**
	* gets the activity of the requesting user for the day
	*
	* @param $day_of_year  Day of Year
	* @param $year        Year
	* @return Harvest_DailyActivity
	*/
	public function getDailyActivity( $day_of_year = null, $year = null ) {

	    $url = "daily/";
	    if( ! is_null( $day_of_year ) && ! is_null( $year ) ) {
	        $url .= $day_of_year . "/" . $year;
	    }
	    return $this->performGet( $url, false );

	}

	/**
	* gets the entry specified
	*
	* @param $entry_id    Entry Identifier
	* @return Harvest_DayEntry
	*/
	public function getEntry( $entry_id ) {

	    $url = "daily/show/" . $entry_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*------------------------- Client API -------------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all clients
	*
	* @return array of Harvest_Client objects
	*/
	public function getClients() {

	    $url = "clients";
	    return $this->performGet( $url, true );

	}

	/**
	* get a single client
	*
	* @param $client_id  Client Identifier
	* @return Harvest_Client
	*/
	public function getClient( $client_id ) {

	    $url = "clients/" . $client_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*---------------------- Client Contacts API -------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all contacts for an account
	*
	* @return array of Harvest_Contact objects
	*/
	public function getContacts() {

	    $url = "contacts";
	    return $this->performGet( $url, true );

	}

	/**
	* get all contacts for a client
	*
	* @param $client_id Client Identifier
	* @return array of Harvest_Contact objects
	*/
	public function getClientContacts( $client_id ) {

	    $url = "clients/" . $client_id . "/contacts";
	    return $this->performGet( $url, true );

	}

	/**
	* get a client contact
	*
	* @param $cotnact_id  Contact Identifier
	* @return Harvest_Contact
	*/
	public function getContact( $contact_id ) {

	    $url = "contacts/" . $contact_id;
	    return $this->performGet( $url, false );
	
	}
	
	/*--------------------------------------------------------------*/
	/*----------------------- Projects API -------------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all projects
	*
	* @return array of Harvest_Project objects
	*/
	public function getProjects() {

	    $url = "projects";
	    return $this->performGet( $url, true );

	}

	/**
	* get a single project
	*
	* @param $project_id  Project Identifier
	* @return Harvest_Project
	*/
	public function getProject( $project_id ) {

	    $url = "projects/" . $project_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*-------------------------- Tasks API -------------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all tasks
	*
	* @return array of Harvest_Task objects
	*/
	public function getTasks() {

	    $url = "tasks";
	    return $this->performGet( $url, true );

	}

	/**
	* get a single task
	*
	* @param $task_id  Task Identifier
	* @return Harvest_Task
	*/
	public function getTask( $task_id ) {

	    $url = "tasks/" . $task_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*------------------------- People API -------------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all users
	*
	* @return array of Harvest_User objects
	*/
	public function getUsers() {

	    $url = "people";
	    return $this->performGet( $url, true );

	}

	/**
	* get a user
	*
	* @param $user_id  User Identifier
	* @return Harvest_User
	*/
	public function getUser( $user_id ) {

	    $url = "people/" . $user_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*------------------------ Expenses API ------------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all expense categories
	*
	* @return array of Harvest_ExpenseCategory objects
	*/
	public function getExpenseCategories() {

	    $url = "expense_categories";
	    return $this->performGet( $url, true );

	}

	/*--------------------------------------------------------------*/
	/*-------------------- Expenses Tracking API -------------------*/
	/*--------------------------------------------------------------*/

	//-- no get api
    //-- expense receipt image

	/*--------------------------------------------------------------*/
	/*--------------------- User Assignment API --------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all user assignments to a given project
	*
	* @param $project_id
	* @return array of Harvest_UserAssignment objects
	*/
	public function getProjectUserAssignments( $project_id ) {

	    $url = "projects/" . $project_id . "/user_assignments";
	    return $this->performGet( $url, true );

	}

	/**
	* get a user assignment
	*
	* @param $project_id    Project Identifier
	* @param $user_assignment_id  User Assignment Identifier
	* @return Harvest_UserAssignment
	*/
	public function getProjectUserAssignment( $project_id, $user_assignment_id ) {

	    $url = "projects/" . $project_id . "/user_assignments/" . $user_assignment_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*--------------------- Task Assignment API --------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all task assignments to a given project
	*
	* @param $project_id    Project Identifier
	* @return array of Harvest_TaskAssignment objects
	*/
	public function getProjectTaskAssignments( $project_id ) {

	    $url = "projects/" . $project_id . "/task_assignments";
	    return $this->performGet( $url, true );

	}

	/**
	* get a task assignment
	*
	* @param $project_id    Project Identifier
	* @param $task_assignment_id  Task Assignment Identifier
	* @return Harvest_TaskAssignment
	*/
	public function getProjectTaskAssignment( $project_id, $task_assignment_id ) {

	    $url = "projects/" . $project_id . "/task_assignments/" . $task_assignment_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*------------------------- Reports API ------------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all project entries for given time range for a particular user if specified
	*
	* @param $project_id    Project Identifier
	* @param $range    Harvest_Range
	* @param $user_id    User identifier optional
	* @return array of Harvest_InvoiceMessage objects
	*/
	public function getProjectEntries( $project_id, Harvest_Range $range, $user_id = null ) {

	    $url = "projects/" . $project_id . "/entries?from=" . $range->from() . '&to=' . $range->to();
	    if( ! is_null( $user_id ) ) {
	        $url .= "&user_id=" . $user_id;
	    }
	    return $this->performGet( $url, true );

	}

	/**
	* get all user entries for given time range and for a particular project if specified
	*
	* @param $user_id    user Identifier
	* @param $range    Harvest_Range
	* @param $project_id    Project identifier optional
	* @return array of Harvest_InvoiceMessage objects
	*/
	public function getUserEntries( $user_id, Harvest_Range $range, $project_id = null ) {

	    $url = "people/" . $user_id . "/entries?from=" . $range->from() . '&to=' . $range->to();
	    if( ! is_null( $project_id ) ) {
	        $url .= "&project_id=" . $projected_id;
	    }
	    return $this->performGet( $url, true );

	}

	/**
	* get all user expenses for given time range
	*
	* @param $user_id    user Identifier
	* @param $range    Harvest_Range
	* @return array of Harvest_InvoiceMessage objects
	*/
	public function getUserExpenses( $user_id, Harvest_Range $range ) {

	    $url = "people/" . $user_id . "/expenses?from=" . $range->from() . '&to' . $range->to();
	    return $this->performGet( $url, true );

	}
	
	/*--------------------------------------------------------------*/
	/*------------------------ Invoices API ------------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all invoices
	*
	* @param $filter    Harvest_Invoice_Filter
	* @return array of Harvest_Invoice objects
	*/
	public function getInvoices( Harvest_Invoice_Filter $filter = null) {

	    //-- need filter class  page, from, to , status, client
		$url = "invoices";
		if( ! is_null( $filter ) ) {
			$url .= $filter->toURL();
		}
	    return $this->performGet( $url, true );
	
	}

	/**
	* get a specific invoice
	*
	* @param $invoice_id  Invoice Identifier
	* @return Harvest_Invoice
	*/
	public function getInvoice( $invoice_id ) {

	    $url = "invoices/" . $invoice_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*-------------------- Invoice Messages API --------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all messages sent for an invoice
	*
	* @param $invoice_id    Invoice Identifier
	* @return array of Harvest_InvoiceMessage objects
	*/
	public function getInvoiceMessages( $invoice_id ) {

	    $url = "invoices/" . $invoice_id . "/messages";
	    return $this->performGet( $url, true );

	}

	/**
	* get a particular invoice message
	*
	* @param $invoice_id    Invoice Identifier
	* @param $message_id      Message Identifier
	* @return Harvest_InvoiceMessage
	*/
	public function getInvoiceMessage( $invoice_id, $message_id ) {

	    $url = "invoices/" . $invoice_id . "/messages/" . $message_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*-------------------- Invoice Payments API --------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all recorded payments for an invoice
	*
	* @param $invoice_id    Invoice Identifier
	* @return array of Harvest_Payment objects
	*/
	public function getInvoicePayments( $invoice_id ) {

	    $url = "invoices/" . $invoice_id . "/payments";
	    return $this->performGet( $url, true );

	}

	/**
	* get a particular invoice payment
	*
	* @param $invoice_id    Invoice Identifier
	* @param $payment_id      Payment Identifier
	* @return Harvest_Payment
	*/
	public function getInvoicePayment( $invoice_id, $payment_id ) {

	    $url = "invoices/" . $invoice_id . "/payments/" . $payment_id;
	    return $this->performGet( $url, false );

	}

	/*--------------------------------------------------------------*/
	/*------------------- Invoice Categories API -------------------*/
	/*--------------------------------------------------------------*/

	/**
	* get all invoice categories
	*
	* @return array of Harvest_InvoiceItemCategory objects
	*/
	public function getInvoiceCategories() {

	    $url = "invoice_item_categories";
	    return $this->performGet( $url, true );

	}

	/*--------------------------------------------------------------*/
	/*----------------- API Access & Parse Methods -----------------*/
	/*--------------------------------------------------------------*/

	/**
	 * perform http get command
	 * @param string $url
	 * @return string xml response
	 */
	private function performGet( $url, $multi = true ) {
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, "http://" . $this->_account . ".harvestapp.com/" . $url );
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Accept: application/xml', 'Content-Type: application/xml', 'Authorization: Basic (' . base64_encode( $this->_user . ":" . $this->_password ). ')' ) );
		$xml = curl_exec($ch);
		$code = curl_getinfo($ch,CURLINFO_HTTP_CODE);
		
		$data = null;
		// valid
        if( "2" == substr( $code, 0, 1 ) ) {
            if( $multi ) {
                $data = $this->parseItems( $xml );
            } else {
                $data = $this->parseItem( $xml );
            } 
        } else {
            //invalid
        }
       
		return new Harvest_Result( $code, $data );
	}

	/**
	 * parse xml list
	 * @param string $xml
	 * @return array of items
	 */
	private function parseItems( $xml ) {
		$items = array();

		$xmlDoc = new DOMDocument();
		$xmlDoc->loadXML($xml);
		$x = $xmlDoc->documentElement;
		
		foreach ($x->childNodes AS $item)
		{
			$item = $this->parseNode( $item );
			if( ! is_null( $item ) ) 
			{
				$items[$item->id()] = $item;
			}
		}
		
		return $items;
		
	}
	
	/**
	 * parse xml item
	 * @param string $xml
	 * @return object
	 */
	private function parseItem( $xml ) {
		$item = null;

		$xmlDoc = new DOMDocument();
		$xmlDoc->loadXML($xml);
		$itemNode = $xmlDoc->documentElement;
		
		return $this->parseNode( $itemNode );
		
	}

	/**
	 * parse xml node
	 * @param documentElement $node
	 * @return object
	 */
	private function parseNode( $node ) {
		$item = null;
		
		switch( $node->nodeName ) 
		{	
			case "category":
				$item = new Harvest_Category();
			break;
			case "client":
				$item = new Harvest_Client();
			break;
			case "contact":
				$item = new Harvest_Contact();
			break;
			case "day-entry":
				$item = new Harvest_DayEntry();
			break;
			case "expense":
				$item = new Harvest_Expense();
			break;
			case "invoice":
				$item = new Harvest_Invoice();
			break;
			case "invoice-item-category":
				$item = new Harvest_InvoiceItemCategory();
			break;
			case "invoice-message":
				$item = new Harvest_InvoiceMessage();
			break;
			case "payment":
				$item = new Harvest_Payment();
			break;
			case "project":
				$item = new Harvest_Project();
			break;
			case "task":
				$item = new Harvest_Task();
			break;
			case "user":
				$item = new Harvest_User();
			break;
			case "user-assignment":
				$item = new Harvest_UserAssignment();
			break;
			case "daily":
				$item = new Harvest_DailyActivity();
			break;
			default:
			break;
		}
		if( ! is_null( $item ) ) {
			$item->parseXML( $node );
		}
		
		return $item;
		
	}

	/**
     * simple autoload function
     * returns true if the class was loaded, otherwise false
     *
     * @param string $classname
     * @return boolean
     */
    public static function autoload($className)
    {
        if (class_exists($className, false) || interface_exists($className, false)) {
            return false;
        }

        $class = self::getPath() . DIRECTORY_SEPARATOR . str_replace('_', DIRECTORY_SEPARATOR, $className) . '.php';

        if (file_exists($class)) {
            require $class;

            return true;
        }

        return false;
    }

	/**
     * Get the root path to Harvest API
     *
     * @return string
     */
    public static function getPath()
    {
        if ( ! self::$_path) {
            self::$_path = dirname(__FILE__);
        }

        return self::$_path;
    }

}
?>